@@ -46,8 +46,7 @@
 #include "panel-gconf.h"
 #include "quick-desktop-reader.h"
 #include "nothing.h"
-
-#define ICON_SIZE 20
+#include "panel-stock-icons.h"
 
 enum {
 	COLUMN_ICON,
@@ -60,11 +59,9 @@ enum {
 };
 
 typedef enum {
-	PANEL_RESPONSE_RUN,
+	PANEL_RESPONSE_RUN
 } PanelResponseType;
 
-#define PANEL_STOCK_RUN "panel-run"
-
 #define ENABLE_LIST_DEFAULT TRUE
 #define SHOW_LIST_DEFAULT FALSE
 
@@ -209,9 +206,11 @@ launch_selected (GtkTreeModel *model,
 		 GtkWidget    *dialog)
 {
 	GnomeDesktopItem *ditem;
+	GnomeEntry	 *gnome_entry;
+	GtkWidget 	 *entry;
 	GError           *error = NULL;
 	GtkToggleButton  *terminal;
-	char             *name;
+	char             *name, *command;
 
 	gtk_tree_model_get (model, iter, COLUMN_NAME, &name, -1);
 
@@ -247,9 +246,15 @@ launch_selected (GtkTreeModel *model,
 				      "Details: %s"),
 				    error->message);
 		g_clear_error (&error);
-		return;
 	}
 
+	/* save command history */
+	gnome_entry = g_object_get_data (G_OBJECT (dialog), "gnome_entry");
+	entry = g_object_get_data (G_OBJECT (dialog), "entry");
+	command = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);	
+	gnome_entry_prepend_history (gnome_entry, TRUE, command);
+	g_free (command);
+	
 	gnome_desktop_item_unref (ditem);
 }
 
@@ -258,6 +263,7 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 {
 	GtkWidget *entry;
         GtkWidget *list;
+	GnomeEntry *gnome_entry;
 	char **argv = NULL;
 	char **temp_argv = NULL;
 	int argc, temp_argc;
@@ -290,6 +296,7 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 
 		launch_selected (model, &iter, w);
         } else {
+	
 		GtkToggleButton *terminal;
 
                 entry = g_object_get_data (G_OBJECT (w), "entry");
@@ -300,6 +307,10 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
                 if (string_empty (s))
                         goto return_and_close;
 
+		/* save command in history */
+		gnome_entry = g_object_get_data (G_OBJECT (w), "gnome_entry");
+		gnome_entry_prepend_history (gnome_entry, TRUE, s);
+
                 /* evil eggies, do not translate! */
                 if (strcmp (s, "you shall bring us a shrubbery") == 0) {
                         panel_info_dialog ("ni_ni_ni_ni",
@@ -542,8 +553,6 @@ browse (GtkWidget *w, GtkWidget *entry)
 		 G_CALLBACK (gtk_widget_destroy),
 		 G_OBJECT (fsel));
 
-	gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
-
 	gtk_window_present (GTK_WINDOW (fsel));
 }
 
@@ -625,6 +634,45 @@ sync_entry_to_list (GtkWidget *dialog)
 	}
 }
 
+static char *
+remove_parameters (const char *exec)
+{
+	GString *str;
+	char    *retval, *p;
+
+	str = g_string_new (exec);
+
+	while ((p = strstr (str->str, "%"))) {
+		switch (p [1]) {
+		case '%':
+			g_string_erase (str, p - str->str, 1);
+			break;
+		case 'U':
+		case 'F':
+		case 'N':
+		case 'D':
+		case 'f':
+		case 'u':
+		case 'd':
+		case 'n':
+		case 'm':
+		case 'i':
+		case 'c':
+		case 'k':
+		case 'v':
+			g_string_erase (str, p - str->str, 2);
+			break;
+		default:
+			break;
+		}
+	}
+
+	retval = str->str;
+	g_string_free (str, FALSE);
+
+	return retval;
+}
+
 static void
 sync_list_to_entry (GtkWidget *dialog)
 {
@@ -665,20 +713,27 @@ sync_list_to_entry (GtkWidget *dialog)
 				gboolean terminal;
                                 const char *exec;
 
-				exec = gnome_desktop_item_get_string
-					(ditem, GNOME_DESKTOP_ITEM_EXEC);
-				if (exec == NULL)
-					exec = gnome_desktop_item_get_string
-						(ditem, GNOME_DESKTOP_ITEM_URL);
-				terminal = gnome_desktop_item_get_boolean
-					(ditem, GNOME_DESKTOP_ITEM_TERMINAL);
+				exec = gnome_desktop_item_get_string (
+						ditem, GNOME_DESKTOP_ITEM_EXEC);
+				if (exec) {
+					char *stripped;
 
-                                gtk_entry_set_text (GTK_ENTRY (entry),
-						    sure_string (exec));
+					stripped = remove_parameters (exec);
 
-                                gtk_toggle_button_set_active
-					(GTK_TOGGLE_BUTTON (terminal_toggle),
-					 terminal);
+					gtk_entry_set_text (GTK_ENTRY (entry), stripped);
+
+					g_free (stripped);
+				} else {
+					exec = gnome_desktop_item_get_string (
+							ditem, GNOME_DESKTOP_ITEM_URL);
+					gtk_entry_set_text (GTK_ENTRY (entry), sure_string (exec));
+				}
+
+				terminal = gnome_desktop_item_get_boolean (
+							ditem, GNOME_DESKTOP_ITEM_TERMINAL);
+
+                                gtk_toggle_button_set_active (
+					GTK_TOGGLE_BUTTON (terminal_toggle), terminal);
 				
                                 gnome_desktop_item_unref (ditem);
                         }
@@ -849,6 +904,7 @@ create_simple_contents (void)
 	gtk_tooltips_set_tip (panel_tooltips, entry, _("Command to run"), NULL);
         gtk_combo_set_use_arrows_always (GTK_COMBO (gentry), TRUE);
         g_object_set_data (G_OBJECT (run_dialog), "entry", entry);
+	g_object_set_data (G_OBJECT (run_dialog), "gnome_entry", gentry);
 
         g_signal_connect (G_OBJECT (entry), "event",
 			  G_CALLBACK (entry_event),
@@ -1084,6 +1140,7 @@ add_icon_idle (GtkListStore *list)
 	gboolean     long_operation = FALSE;
 	GdkPixbuf   *pixbuf;
 	char        *file;
+	int          icon_height;
 
 	do {
 		if (add_icon_paths == NULL) {
@@ -1107,8 +1164,10 @@ add_icon_idle (GtkListStore *list)
 		gtk_tree_model_get (GTK_TREE_MODEL (list), &iter,
 				    COLUMN_ICON_FILE, &file, -1);
 
-		pixbuf = panel_make_menu_icon (file, NULL /* fallback */,
-					       ICON_SIZE, &long_operation);
+		if (!gtk_icon_size_lookup (panel_menu_icon_get_size (), NULL, &icon_height))
+			icon_height = PANEL_DEFAULT_MENU_ICON_SIZE;
+
+		pixbuf = panel_make_menu_icon (file, NULL, icon_height, &long_operation);
 		if (pixbuf) {
 			gtk_list_store_set (list, &iter, COLUMN_ICON, pixbuf, -1);
 			g_object_unref (pixbuf);
@@ -1185,11 +1244,11 @@ fill_list (GtkWidget *list)
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter,
 				    COLUMN_ICON, NULL,
-				    COLUMN_ICON_FILE, fr->icon,
-				    COLUMN_FULLNAME, fr->fullname,
-				    COLUMN_COMMENT, fr->comment,
-				    COLUMN_NAME, fr->name,
-				    COLUMN_EXEC, fr->exec,
+				    COLUMN_ICON_FILE, (fr->icon) ? fr->icon : "",
+				    COLUMN_FULLNAME, (fr->fullname) ? fr->fullname : "",
+				    COLUMN_COMMENT, (fr->comment) ? fr->comment : "",
+				    COLUMN_NAME, (fr->name) ? fr->name : "",
+				    COLUMN_EXEC, (fr->exec) ? fr->exec : "",
 				    -1);
 
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (store), &iter);
@@ -1393,8 +1452,9 @@ create_program_list_contents (void)
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list), FALSE);
 
-	add_atk_name_desc (list, _("List of known applications"),
-			   _("Choose an application to run from the list"));
+	panel_set_atk_name_desc (list,
+				 _("List of known applications"),
+				 _("Choose an application to run from the list"));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -1450,12 +1510,14 @@ update_contents (GtkWidget *dialog)
                 if (program_list_box && program_list_box->parent == NULL) {
 			GtkWidget *list;
 			
+			gtk_window_resize (GTK_WINDOW (dialog), 100, 300);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
-
+			
                         gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
                                             program_list_box,
                                             TRUE, TRUE, GNOME_PAD_SMALL);
                 
+
                         gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
 
 			list = g_object_get_data (G_OBJECT (dialog), "program_list");
@@ -1476,33 +1538,6 @@ update_contents (GtkWidget *dialog)
         }
 }
 
-static inline void
-register_run_stock_item (void)
-{
-	static gboolean registered = FALSE;
-
-	if (!registered) {
-		GtkIconFactory      *factory;
-		GtkIconSet          *execute_icons;
-
-		static GtkStockItem  run_item [] = {
-			{ PANEL_STOCK_RUN, N_("_Run"), 0, 0, GETTEXT_PACKAGE },
-		};
-
-		execute_icons = gtk_icon_factory_lookup_default (GTK_STOCK_EXECUTE);
-
-		factory = gtk_icon_factory_new ();
-
-		gtk_icon_factory_add (factory, PANEL_STOCK_RUN, execute_icons);
-
-		gtk_icon_factory_add_default (factory);
-
-		gtk_stock_add_static (run_item, 1);
-
-		registered = TRUE;
-	}
-}
-
 static void
 run_dialog_destroyed (GtkWidget *widget)
 {
@@ -1543,8 +1578,6 @@ show_run_dialog (void)
 		return;
 	}
 
-	register_run_stock_item ();
-
 	run_dialog = gtk_dialog_new_with_buttons (_("Run Program"),
 						  NULL /* parent */,
 						  0 /* flags */,
@@ -1557,7 +1590,7 @@ show_run_dialog (void)
 	gtk_window_set_resizable (GTK_WINDOW (run_dialog), FALSE);
 
 	w = gtk_dialog_add_button (GTK_DIALOG (run_dialog),
-				   PANEL_STOCK_RUN, PANEL_RESPONSE_RUN);
+				   PANEL_STOCK_EXECUTE, PANEL_RESPONSE_RUN);
 	gtk_widget_set_sensitive (w, FALSE);
 	g_object_set_data (G_OBJECT (run_dialog), "run_button", w);
 
@@ -1572,7 +1605,6 @@ show_run_dialog (void)
 			  G_CALLBACK (run_dialog_destroyed),
 			  NULL);
 
-	gtk_window_set_position (GTK_WINDOW (run_dialog), GTK_WIN_POS_MOUSE);
 	gtk_window_set_wmclass (GTK_WINDOW (run_dialog), "run_dialog", "Panel");
 
 	gtk_dialog_set_default_response (GTK_DIALOG (run_dialog), PANEL_RESPONSE_RUN);
@@ -1609,6 +1641,7 @@ void
 show_run_dialog_with_text (const char *text)
 {
 	GtkWidget *entry;
+	char *exec;
 
 	g_return_if_fail (text != NULL);
 
@@ -1620,5 +1653,9 @@ show_run_dialog_with_text (const char *text)
         
 	entry = g_object_get_data (G_OBJECT (run_dialog), "entry");
 
-	gtk_entry_set_text (GTK_ENTRY (entry), text);
+	exec = remove_parameters (text);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), exec);
+
+	g_free (exec);
 }
